@@ -311,7 +311,7 @@ Switching this off makes copying (by killing) easier for some."
   :type 'boolean
   :group 'lui)
 
-(defcustom lui-max-buffer-size nil
+(defcustom lui-max-buffer-size 102400
   "Non-nil if Lui should truncate the buffer if it grows too much.
 If the buffer size (in characters) exceeds this number, it is
 truncated at the top."
@@ -469,6 +469,8 @@ It can be customized for an application by specifying a
   (lui-time-stamp-enable-filtering)
   (tracking-mode 1)
   (auto-fill-mode 0)
+  (when (fboundp 'cursor-intangible-mode)
+    (cursor-intangible-mode 1))
   (when lui-flyspell-p
     (require 'flyspell)
     (lui-flyspell-change-dictionary)))
@@ -593,7 +595,7 @@ This uses `lui-buttons-list'."
       (goto-char (point-min))
       (while (re-search-forward regex nil t)
         ;; Ensure we're not inserting a button inside a URL button
-        (when (not (button-at (point)))
+        (when (not (button-at (match-beginning 0)))
           (let* ((function (if (functionp function-or-url)
                                function-or-url
                              'browse-url))
@@ -862,19 +864,14 @@ If it is \"\", disable flyspell."
                       (and (fboundp 'ispell-valid-dictionary-list)
                            (mapcar 'list (ispell-valid-dictionary-list)))
                       nil t)))
-  (cond
-   ((not (fboundp 'flyspell-mode))
-    (error "Flyspell mode is not loaded"))
-   ((string= dictionary "")
-    (flyspell-mode 0))
-   (t
-    (let ((dictionary (or dictionary
-                          (lui-find-dictionary (buffer-name)))))
-      (when dictionary
-        (when (or (not (boundp 'flyspell-mode))
-                  (not flyspell-mode))
-          (flyspell-mode 1))
-        (ispell-change-dictionary dictionary))))))
+  (let ((dictionary (or dictionary
+                        (lui-find-dictionary (buffer-name)))))
+    (when flyspell-mode
+      (flyspell-mode 0))
+    (when (and dictionary
+               (not (equal dictionary "")))
+      (ispell-change-dictionary dictionary))
+    (flyspell-mode 1)))
 
 
 (defun lui-find-dictionary (buffer-name)
@@ -960,17 +957,18 @@ The positions are adjusted by SHIFT positions."
   ;; Thanks to ERC for the idea of the code.
   ;; ERC's code doesn't take care of an OLD-BEGIN value, which is
   ;; necessary if you allow modification of the buffer.
-  (let* ((adjust-position (lambda (pos)
-                            (if (and (numberp pos)
-                                     ;; After the boundary: Adjust
-                                     (>= (abs pos)
-                                         old-begin))
-                                (* (if (< pos 0)
-                                       -1
-                                     1)
-                                   (+ (abs pos)
-                                      shift))
-                              pos)))
+  (let* ((gc-cons-threshold most-positive-fixnum)  ;; See debbugs#22120#47
+         (adjust-position (lambda (pos)
+                               (if (and (numberp pos)
+                                        ;; After the boundary: Adjust
+                                        (>= (abs pos)
+                                            old-begin))
+                                   (* (if (< pos 0)
+                                          -1
+                                        1)
+                                      (+ (abs pos)
+                                         shift))
+                                 pos)))
          (adjust (lambda (entry)
                    (cond
                     ;; POSITION
@@ -1227,7 +1225,8 @@ information.")
                            'lui-time-stamp t))
                (start (point)))
           (insert ts-string)
-          (add-text-properties start (1+ (point)) '(intangible t)))))
+          (add-text-properties start (1+ (point)) '(intangible t))
+          (add-text-properties (1+ start) (point) '(cursor-intangible t)))))
      ;; Time stamps left
      ((eq lui-time-stamp-position 'left)
       (let ((indent-string (propertize (make-string (length ts) ?\s)
